#from http://www-stat.stanford.edu/~tibs/ElemStatLearn/data.html
waveform <- function(n)
{
  class <- as.numeric(cut(runif(n), c(0, 1/3, 2/3, 1)))
  h <- function(xoff)
    pmax(6 - abs(seq(21) - 11 + xoff), 0)
  x <- rbind(h(0), h(-4), h(4))
  x1 <- x[c(1, 1, 2),  ][class,  ]
  x2 <- x[c(2, 3, 3),  ][class,  ]
  u <- runif(n)
  data.frame(x = I(u * x1 + (1 - u) * x2 + rnorm(n * 21)), y = class)
}

#modify waveform in a binary problem by removing
#the instances for the third class value
waveform.binary <- function(n){
  tmp <- waveform(trunc(1.75*n))
  output <- subset(tmp, tmp$y != 3)
  if (nrow(output) > n){
    output <- output[1:n,]
  }
  output$y <- factor(output$y)
  levels(output$y) <- c("A","B")
  return(output)
}

#add K random attributes
add.rnd <- function(wave.data, K = 1){
 n <- nrow(wave.data)
 new.wave <- wave.data
 for (k in 1:K){
  colref <- trunc(runif(1,min=1,max=22))
  newcol <- rnorm(n,mean=mean(wave.data$x[,colref]),sd=sd(wave.data$x[,colref]))
  newcol <- as.numeric(newcol)
  new.wave <- cbind(new.wave,newcol)
  names(new.wave)[ncol(new.wave)] <- paste("rnd",k,sep="_")
 }
 return(new.wave)
}

#add L correlated attributes
add.correlated <- function(wave.data,L = 1, noise=1){
 n <- nrow(wave.data)
 new.wave <- wave.data
 for (l in 1:L){
  colref <- trunc(runif(1,min=1,max=22))
  newcol <- wave.data$x[,colref] + rnorm(n,0,sd=sd(wave.data$x[,colref])*noise)
  newcol <- as.numeric(newcol)
  new.wave <- cbind(new.wave,newcol)
  names(new.wave)[ncol(new.wave)] <- paste("cor",l,sep="_")
 }
 return(new.wave)
}

#function for rounding numeric columns
myround <- function(x){
  if (is.factor(x)==T){
  return(x)
  } else
  {
  return(round(x,3))
  }
}

#generate n instances with K rnd attributes and L correlated attributes
generate.binary <- function(n,K=1,L=1,noise=1){
  tmp <- waveform.binary(n)
  tmp <- add.rnd(tmp,K)
  tmp <- add.correlated(tmp,L,noise)
  output <- cbind(as.data.frame(matrix(tmp$x,nrow(tmp),21)),subset(tmp,select=-x))
  output <- as.data.frame(lapply(output,myround))
  return(output)
}

#generate and save a dataset
set.seed(1)
dataset.size <- 300000 #number of instances
nb.rnd <- 50 #number of random variables
nb.cor <- 50 #number of correlated variables
noise.level <- 1 #noise for correlated variables
data.wave <- generate.binary(dataset.size,nb.rnd,nb.cor,noise.level)
summary(data.wave)
#writting
write.table(data.wave,file="wavebin.txt",quote=F,sep="\t",row.names=F)


